// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Common_generic_commands.thrift
 *
 * CHANGELOG
 * 04 nov 2015  - Create file from MA5G and CIM generic commands
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 29 Dec 2016  - Update file_load command comment
 * 24 Mar 2017  - PRGCA Product support added
 * 02 Jul 2018  - MA_VP_MD Product support added
 */
// </doc_history>

include "Cls_types.thrift"
include "Generic_types.thrift"
include "File_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

service Common_generic_commands
{
    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Read files from a contactless card.
     *
     *
     * Return the list of read files/data
     */
    list<binary> cls_read(  /** Duration max. of the operation in seconds*/
                            1: i32                              timeout_in_sec,
                            /** List of card to support*/
                            2: Cls_types.Cls_cards_definition   cards)
            throws (    /** Timeout has been reached*/
                        1: Generic_types.Timeout_error          ex1,
                        /** Invalid argument*/
                        2: Generic_types.Invalid_argument_error ex2,
                        /** One of the specified contactless key is invalid*/
                        3: Cls_types.Cls_invalid_key_error      ex3,
                        /** Generic exception */
                        4: Generic_types.Generic_error generic_xcept);

    /**
     * Erase a file represented by name and type
     */
    void file_erase(    1: File_types.File_details file_details)
        throws( /** The specified file does not exist or insufficient data provided */
                1: Generic_types.File_inexistent_error  file_inexistent_xcept
                /** The SD card not found (might be required for specific files) */
                2: Generic_types.SD_card_not_found  SD_card_not_found_xcept
                /** Generic exception */
                3: Generic_types.Generic_error  generic_xcept
                /** Invalid parameter*/
                4: Generic_types.Invalid_argument_error invalid_argument_xcept);

        /**
     * Load a file into the terminal.<br>
     * <b>Note:</b>
     * <ol>
     *   <li>The <code>File_types.File_chunk.action</code> shall be taken into consideration only for the last chunk,
     *     i.e., when <code>File_types.File_chunk.is_last</code> is true.
     *   <li>Maximum file sizes:
     *     <ul>
     *       <li><code>File_types.File_type.video</code>:   10MB product_video_file_note
     *       <li><code>File_types.File_type.audio</code>:   500 KB
     *       <li><code>File_types.File_type.picture</code>:   1 MB
     *       <li>Others:                                      5 MB
     *     </ul>
     *   <li>Maximum chunk size is 10240 bytes, i.e. 10 KB
     *   <li>Make sure video file is in proper format. (MPEG4 or VP8)
     *   <li>While updating stolen card entries, all previous entries will be deleted
     * </ol><br>
     * <b>Warning:</b><br>
     * <ol>
     *   <li>You shall need a modified version of the generated code that supports
     *       handling of several request/response messages for the single call to the command.
     *   <li>For file type <code>File_types.File_type.stolen_card_list</code>, No error will be return.
     *       And maximum 250000 valid stolen card entries will be stored and rest of the entries will be ignored.
     * </ol><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     * void Generic_commandsClient::file_load(const  ::Distant_cmd::File_details& file_details, const  ::Distant_cmd::File_chunk& chunk)
     * {
     *     <span style="color:green">// Original code section in the generated code
     *     // send_file_load(file_details, chunk);
     *     // recv_file_load();</span>
     * &nbsp;
     *     <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data
     *     // This is actually multi-request handling</span>
     *     ::Distant_cmd::File_chunk actual_chunk; <span style="color:green">// a non-const chunk required</span>
     *     while(true)
     *     {
     *         <span style="color:green">// Client's call-back is supposed to load the data that shall be available in the 'actual_chunk' parameter to send to the Thrift server.
     *         //  Note that in case of the last chunk of data to be uploaded the 'actual_chunk.is_last' flag must be set.</span>
     *         Multiresponse_handler::client_cb_file_load(actual_chunk);
     *         send_file_load(file_details, actual_chunk);
     * &nbsp;
     *         recv_file_load(); <span style="color:green">// required to see if exception was received -- so that further sending is stopped</span>
     * &nbsp;
     *         if(actual_chunk.is_last) <span style="color:green">// Exit the loop since the last chunk has been sent and its corresponding response received</span>
     *         {
     *             break;
     *         }
     *     }
     *     <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     * }
     * </pre>
     */
    void file_load( /** In cases where file name is required, the file shall be uploaded with the same name as the file name provided */
                    1: File_types.File_details  file_details,
                    /** The chunk of the file */
                    2: File_types.File_chunk    chunk)
        throws( /** The specified file does not exist or insufficient data provided */
                1: Generic_types.File_inexistent_error  file_inexistent_xcept,
                /** The specified file or chunk exceeds its maximum limit */
                2: Generic_types.File_too_large_error   file_large_xcept,
                /** The file-name is missing */
                3: Generic_types.File_name_missing_error    file_name_missing_xcept,
                /** The particular type of file with the specified extension is not allowed */
                4: Generic_types.Invalid_file_extension_error   invalid_file_ext_xcept,
                /** The SD card not found (might be required for specific files) */
                5: Generic_types.SD_card_not_found  SD_card_not_found_xcept
                /** Generic exception */
                6: Generic_types.Generic_error  generic_xcept
                /** Invalid parameter*/
                7: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Capture a picture in specified format
     *
     *
     * The terminal will capture a picture from camera or will perform a screenshot,
     * and will return the picture in specified format.
     *
     * Return the raw data of the captured picture
     */
    binary picture_capture( /** Interface that will capture the picture (Cameras or screen) */
                            1: Generic_types.Picture_interface input_interface,
                            /** Format of the picture to retrieve (JPEG, BMP ...) */
                            2: Generic_types.Picture_format format)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept
                    /** Invalid argument */
                    2: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Display a picture in specified format
     *
     *
     * The terminal will display a picture on the screen.<br>
     * The picture will start at the specified coordinates and will be displayed
     * during the specified duration
     */
    void picture_display(   /** Format of the picture to display (JPEG, BMP ...) */
                            1: Generic_types.Picture_format format
                            /** Raw data of the picture file */
                            2: binary picture_data,
                            /** Screen coordinates of the right-upper corner of the picture */
                            3: Generic_types.XY_coordinates coordinates,
                            /** Duration in seconds during which the picture is displayed */
                            4: i32 duration_in_sec)
        throws( /** Invalid argument */
                1: Generic_types.Invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Get product information from terminal board and sensor board.
     *
     * Return requested information
     */
    Generic_types.Product_info product_get_info(   /** List of requested information */
                                                   1: set<Generic_types.Product_info_type> info_type)
        throws  (   /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Get Firmware versions
     *
     * Return Firmware version of requested board type (Terminal or Sensor)
     */
    string terminal_get_version(    /** Board to retrieve Firmware version from */
                                    1: Generic_types.Generic_firmware_type firmware_type)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * When the command returns, terminal is going to reboot
     */
    void terminal_reboot()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Send back a received buffer
     *
     * Buffer shall be less than 64 bytes
     *
     * Return the received buffer
     */
    binary terminal_echo(   /** Buffer to be repeated */
                            1: binary buffer)
        throws( /** The input parameter's size is over size limit (64 bytes) */
                1: Generic_types.Out_of_memory_error    out_of_memory_xcept
                /** Generic exception */
                2: Generic_types.Generic_error          generic_xcept);

    /**
     * Retrieve the list of terminal internal peripherals.
     */
    list<Generic_types.Terminal_peripherals> terminal_retrieve_peripherals()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * That functions is used to configure:
     * <ol>
     *   <li> Terminal's local date and time (i.e. not Network Time Protocol) </li>
     *   <li> Terminal's IP configuration </li>
     *   <li> Terminal's WIFI configuration </li>
     *   <li> Terminal's serial channel configuration </li>
     * </ol>
     * <b>Note:</b>
     * <ul>
     *   <li>While setting the serial channel configuration, a delay of 3 seconds is provided after sending the response
     *       to the client, so that the configuration is applied only after the response is sent to the client.
     * </ul>
     */
    void terminal_set_configuration(    /**
                                         * Structure that contains configuration to apply. <br>
                                         * It can contain several configurations.
                                         */
                                        1: Generic_types.Terminal_configuration terminal_config)
        throws (    /** Invalid parameter set */
                    1: Generic_types.Invalid_argument_error invalid_arg_xcept,
                    /** Failed to configure Ethernet settings */
                    2: Generic_types.Ethernet_configuration_failed ethernet_config_xcept,
                    /** WIFI module not connected */
                    3: Generic_types.WIFI_module_not_connected WIFI_mod_xcept,
                    /** License not found exception */
                    4: Generic_types.License_not_found license_not_xcept,
                    /** Generic exception */
                    5: Generic_types.Generic_error generic_xcept);
}
